;Qt interface driver for lookout

(dynamic-call "scm_init_qt_look" (dynamic-link "libguile-qtlook.so"))

(use-modules (alterator lookout common)
             (alterator object))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;filter: extract lowlevel qt widgets from requests
(define (qt-widget-extractor widget)
  (define-operation get-widget)
  (lambda args
    (apply widget (map (lambda(x)
			 (cond 
			       ((or (is-a? x <widget-holder>)
			            (is-a? x <id-holder>))
			            (x 'get-widget))
			       ((is-a? x <widget>) (get-widget x))
			       (else x)))
		       args))))


(define (make-qt-default-widget widget)
  (define (simplify value) 
    (if (and (pair? value) (= (length value) 1))
      (car value)
      value))
  (object
    <qt-default-widget>
    ((get self name . addon) (widget-get-attr widget (sure-string name) addon))
    ((set self name . value) (widget-set-attr widget
				              (sure-string name)
					      (simplify value)))))

(define (make-qt-dialog widget)
  (join
     <qt-dialog>
    (object
      <qt-basic-dialog>
      ((start self) (widget-start widget))
      ((stop self) (widget-stop widget)))
    (make-qt-default-widget widget)))

(define (make-qt-listbox-setter widget default)
  (define-operation append-item)
  (define-operation remove)
  (define-operation set)
  (object
    <qt-listbox-setter>
    ((items self itemlist) (remove self 'all)
			   (map (lambda (item) (append-item self item)) itemlist))
     ((append-item self item) (widget-set-attr widget "append-item" item))
     ((remove self . range)  (if (eq? range 'all)
			     (widget-set-attr widget "remove" 'all)
			     (for-each (lambda(number) (widget-set-attr widget "remove" number))
					(reverse (extract-range range)))))
     ((item-pixmap self pixmap . range) (for-each (lambda(number) (widget-set-attr
     								   widget
								   "item-pixmap" (cons number pixmap)))
						 (extract-range range)))
     ((item-text self text . range) (for-each (lambda (number) (widget-set-attr
     								widget
								"item-text" (cons number text)))
					     (extract-range range)))
     ((<qt-listbox-setter> self . args) (apply set default args))))

(define (make-qt-listbox widget)
  (define-operation set)
  (let* ((default-widget (make-qt-default-widget widget))
	 (qt-listbox-setter (make-qt-listbox-setter widget default-widget)))
    (join
      <qt-listbox>
      (object
	<qt-basic-listbox>
	((set self value . args) (apply (callable qt-listbox-setter) value args)))
      default-widget)))


(define (make-qt-widget constructor . rest)
  (lambda (eventholder parent-widget)
    (let ((callable-holder (callable eventholder)))
      (let* ((widget (constructor  callable-holder (and parent-widget (parent-widget 'get-widget))))
	     (mixed-object (if (null? rest)
			     (make-qt-default-widget widget)
			     ((car rest) widget))))
	(qt-widget-extractor
	  (join
	    <qt-widget>
	    (object
	      <qt-basic>
	      ((get-widget self) widget)
	      ((delete self) (delete-widget widget)))
	    mixed-object))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;driver description
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(declare-widget dialog (make-qt-widget dlg-make-dialog make-qt-dialog))
(declare-widget button (make-qt-widget dlg-make-button) text)
(declare-widget label (make-qt-widget dlg-make-label) text)
(declare-widget edit (make-qt-widget dlg-make-lineedit) text)
(declare-widget vbox (make-qt-widget dlg-make-vbox))
(declare-widget hbox (make-qt-widget dlg-make-hbox))
(declare-widget splashscreen (make-qt-widget dlg-make-splashscreen))
(declare-widget checkbox (make-qt-widget dlg-make-checkbox) text)
(declare-widget groupbox (make-qt-widget dlg-make-groupbox) title)
(declare-widget progressbar (make-qt-widget dlg-make-progressbar))
(declare-widget textbox (make-qt-widget dlg-make-textbox) text)
(declare-widget scrollview (make-qt-widget dlg-make-scrollview))
(declare-widget listbox (make-qt-widget dlg-make-listbox make-qt-listbox))
(declare-widget combobox (make-qt-widget dlg-make-combobox make-qt-listbox))
(declare-widget tabbox (make-qt-widget dlg-make-tabbox))

;wrapper over tab-panel
(define (tab tab-label . args)
  (make-widget-holder (lambda(parent)
			(let ((root (make-widget lookout-root-widget parent)))
			  (widget-parse-args (callable root) args)
			  (parent 'set 'tab-panel tab-label root)
			  root))))

(declare-application-constructor make-qt-application)
(declare-application-destructor delete-qt-application)
(declare-application-runner (lambda() (document:popup "/")))

(declare-popup-widget (make-qt-widget dlg-make-dialog make-qt-dialog))
(declare-root-widget (make-qt-widget dlg-make-vbox))

